import Acquisition
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName
from slc.linkcollection.browser.viewlets import LinkBoxViewlet

class LinkcollectionView(BrowserView, LinkBoxViewlet):
    """View for displaying the contents of a Linkcollection, e.g. on a folder
    """
    template = ViewPageTemplateFile('linkcollection_view.pt')
    template.id = "linkcollection-view"

    def __call__(self):

        return self.template()



from zope import interface, schema, component
#from z3c.form import form, field, button
#from plone.z3cform.layout import wrap_form
from DateTime import DateTime
from persistent import Persistent

from zope.app.form import CustomWidgetFactory
from zope.app.component.hooks import getSite
from zope.annotation import factory
from zope.annotation.interfaces import IAnnotations, IAttributeAnnotatable, IAnnotatable
from zope.component import adapts
from zope.formlib import form
from zope.interface import Interface, implements
from zope.interface.common import idatetime

from Products.ATContentTypes.interface import IATDocument, IATFolder
from Products.CMFCore.utils import getToolByName

from plone.app.form.widgets.uberselectionwidget import UberSelectionWidget, UberMultiSelectionWidget
from plone.app.vocabularies.catalog import SearchableTextSourceBinder

from slc.linkcollection.interfaces import ILinkList, ILinkListDocument, ILinkListFolder
from slc.linkcollection import LinkCollectionMessageFactory as _


class LinkListBase(Persistent):
    implements(ILinkList)

    @property
    def portal_catalog(self):        
        """ make the adapter penetratable for the vocabulary to find the cat"""
        return getToolByName(getSite(), 'portal_catalog')

    @property
    def portal_url(self):        
        """ make the adapter penetratable for the vocabulary to find the cat"""
        return getToolByName(getSite(), 'portal_url')
        
    urls = []
  
    
class LinkList(LinkListBase):
    implements(ILinkList, ILinkListDocument)
    adapts(IATDocument)

linklist_adapter_document = factory(LinkList)

class LinkListFolder(LinkListBase):
    implements(ILinkList, ILinkListFolder)
    adapts(IATFolder)

linklist_adapter_folder = factory(LinkListFolder)

class LinkCollectionForm(form.PageEditForm):
    form_fields = form.Fields(ILinkList)
    label = u"Add Content Objects to point to"
    form_fields['urls'].custom_widget = UberMultiSelectionWidget

    @form.action(_("Apply"))
    def handle_edit_action(self, action, data):
        ILinkList(self.context).urls = data['urls']

        status = _("Updated on ${date_time}",
                   mapping={'date_time': DateTime()}
                   )
        self.status = status

LinkCollectionView = LinkCollectionForm




import os
import unittest

from zope.testing import doctest
from zope.component import testing
from Products.PloneTestCase import PloneTestCase
from Products.PloneTestCase.layer import PloneSite
from Testing.ZopeTestCase import FunctionalDocFileSuite as Suite
from Testing import ZopeTestCase as ztc
from Products.PloneTestCase.layer import onsetup
from Globals import package_home
from slc.linkcollection import GLOBALS


class LinkCollectionFunctionalTestCase(PloneTestCase.PloneTestCase):
    pass

OPTIONFLAGS = (doctest.REPORT_ONLY_FIRST_FAILURE |
               doctest.ELLIPSIS |
               doctest.NORMALIZE_WHITESPACE)

PloneTestCase.setupPloneSite()

def test_suite():
    home = package_home(GLOBALS)
    print home + '/README.txt',
    return unittest.TestSuite([

        Suite(
           'README.txt',
           optionflags=OPTIONFLAGS,
           package='slc.linkcollection',
           test_class=LinkCollectionFunctionalTestCase
           ),

        ])



from zope.i18nmessageid import MessageFactory
LinkCollectionMessageFactory = MessageFactory('slc.linkcollection')
GLOBALS = globals()

def initialize(context):
    """Initializer called when used as a Zope 2 product."""



from Acquisition import aq_inner

from zope import schema
from zope.app.component.hooks import getSite
from zope.interface import Interface

from Products.ATContentTypes.interface import IATDocument
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces import IPloneSiteRoot
from plone.app.vocabularies.catalog import SearchableTextSource, parse_query
from plone.app.vocabularies.catalog import SearchableTextSourceBinder


class ILinkCollectionLayer(Interface):
    """A layer specific to LinkCollection
    """

class ILinkListDocument(Interface):
    """ marker interface for Linklists on documents"""

class ILinkListFolder(Interface):
    """ marker interface for Linklists on folders"""


class LocalSearchableTextSource(SearchableTextSource):
    """
    Overriding the search method since it adds the current path to
    the results even though it may be a Folder.
    """
    def __init__(self, context, base_query={}, default_query=None):
        super(LocalSearchableTextSource,
              self).__init__(context,
                             base_query=base_query,
                             default_query=default_query)

    def search(self, query_string):
        query = self.base_query.copy()
        if query_string == '':
            if self.default_query is not None:
                query.update(parse_query(self.default_query, self.portal_path))
            else:
                return []
        else:
            query.update(parse_query(query_string, self.portal_path))
        try:
            results = (x.getPath()[len(self.portal_path):]
                       for x in self.catalog(**query))
        except ParseError:
            return []
        return results


class LocalSearchableTextSourceBinder(SearchableTextSourceBinder):
    """ make the binder search in the local folder first """

    def __call__(self, context):
        portal_url = getToolByName(context, 'portal_url', None)
        site = getSite()
        if IPloneSiteRoot.providedBy(site):
            if ILinkListFolder.providedBy(context):
                idx = 0
            else:
                idx = 1
            current_path = '/'+'/'.join(
                portal_url.getRelativeContentPath(site.REQUEST.PARENTS[idx])
                )
            self.default_query = 'path:%s' % current_path
        else:
            self.default_query = 'path:'
        return LocalSearchableTextSource(context, base_query=self.query.copy(),
                                    default_query=self.default_query)


class ILinkList(Interface):
    urls = schema.List(
            title=u"Referenced Documents",
            description=\
u"""Search and select the documents you want to add to your
linklist. The first box contains your current selection. Below it you
can do a fulltext search for documents. Below the search are the
search results you can pic from. Initially they show the contents of
the current folder.""",
            required=True,
            value_type=schema.Choice(
                        title=u"Add documents for referencing",
                        source=LocalSearchableTextSourceBinder(
                {'object_provides':IATDocument.__identifier__}
                                )
                        )
            )



from zope.app.component.hooks import getSite

from Products.ATContentTypes.interface import IATDocument, IATFolder
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from plone.app.layout.viewlets import common

from slc.linkcollection.interfaces import ILinkList
from types import *

class LinkBoxViewlet(common.ViewletBase):

    render = ViewPageTemplateFile('linkbox.pt')
    
    def showeditbox(self):
        user = getToolByName(self.context, 'portal_membership').getAuthenticatedMember()
        if user.has_permission('Modify portal content', self.context):
            return True
        return False
    
    def show(self):
        user = getToolByName(self.context, 'portal_membership').getAuthenticatedMember()
        if not IATDocument.providedBy(self.context) and not IATFolder.providedBy(self.context):
            return False
        if user.has_permission('Modify portal content', self.context):
            return True
        if self.links() != []:
            return True
        return False
            
    def name_item(self, link):
        idx = self.links().index(link)
        return 'linklist-item-%s' % idx
        
    def raw(self):
        if not IATDocument.providedBy(self.context) and not IATFolder.providedBy(self.context):
            return []
        urls = ILinkList(self.context).urls
        return urls
        
    def links(self):
        if not IATDocument.providedBy(self.context) and not IATFolder.providedBy(self.context):
            return []
        urls = ILinkList(self.context).urls
        if not urls:
            return []
        portal = getSite()
        maps = []
        if type(urls) not in (ListType, TupleType):
            urls = [urls]
        for url in urls:
            if url.startswith('/'):
                url = url[1:]
            ob = portal.restrictedTraverse(url, None)
            
            if ob is not None:
                maps.append(dict(title=ob.Title(), url=url, uid=ob.UID()))
                
        return maps

    def docs(self, links=None):
        """ Return the Pages (IATDocument) being linked to.
        """
        docs = []
        links = links or self.links()
        portal = getToolByName(self.context, 'portal_url').getPortalObject()
        for link in links:
            doc = portal.restrictedTraverse(link['url'], None)
            if doc is not None:
                docs.append(doc)
        return docs

    def docbody(self, doc):
        """ Return the document's contents as html
        """
        title = unicode(doc.Title(), 'utf-8')
        description = unicode(doc.Description(), 'utf-8')
        bodytext = unicode(doc.getText(), 'utf-8')
        text = """<h1 class="documentFirstHeading">%s</h1>
                  <div class="documentDescription">%s</div>%s
                """ % (title, description, bodytext)
        return text
        
        


        




# -*- coding: utf-8 -*-
"""
This module contains the slc.linkcollection package
"""
import os
from setuptools import setup, find_packages

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

version = '1.3.2'

long_description = (
    read('README.txt')
    + '\n' +
    'Change history\n'
    '**************\n'
    + '\n' +
    read('CHANGES.txt')
    + '\n' +
    'Detailed Documentation\n'
    '**********************\n'
    + '\n' +
    read('slc', 'linkcollection', 'README.txt')
    + '\n' +
    'Contributors\n'
    '************\n'
    + '\n' +
    read('CONTRIBUTORS.txt')
    + '\n'
    )

tests_require=['zope.testing']


setup(name='slc.linkcollection',
      version=version,
      description="A viewlet displaying links to subdocuments and quickly loading their content via jquery",
      long_description=long_description,
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='viewlet links jquery subnavigation',
      author='Syslab.com GmbH',
      author_email='info@syslab.com',
      url='http://svn.plone.org/svn/plone/plone.example',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['slc'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
	  'plone.browserlayer',
	  
      ],
      tests_require=tests_require,
      extras_require=dict(tests=tests_require),
      test_suite = 'slc.linkcollection.tests.test_docs.test_suite',
      entry_points="""
      # -*- entry_points -*-
      """,
      )

